# Copyright 2020-present Lenovo
# Confidential and Proprietary

import colorsys
import os
import random
import sys

import cv2
import numpy as np
import tensorflow as tf

# Letrain installation path is fixed
letrain_path = "/opt/letrain"
path_list = [
    os.path.join(letrain_path, "applications"),
    os.path.join(letrain_path, "applications/yolov3")
]
sys.path.extend(path_list)

tf.app.flags.DEFINE_bool(
    'inference', True, 'Select whether to make the inference.'
)
tf.app.flags.DEFINE_string(
    "checkpoint_dir", "checkpoint", ""
)
tf.app.flags.DEFINE_string('dataset_types', 'jeg', 'The name of the dataset.')

FLAGS = tf.app.flags.FLAGS

import time


class Timer(object):
    """A simple timer."""
    def __init__(self):
        self.total_time = 0.
        self.calls = 0
        self.start_time = 0.
        self.diff = 0.
        self.average_time = 0.

    def tic(self):
        # using time.time instead of time.clock because time time.clock
        # does not normalize for multithreading
        self.start_time = time.time()

    def toc(self, average=True):
        self.diff = time.time() - self.start_time
        self.total_time += self.diff
        self.calls += 1
        self.average_time = self.total_time / self.calls
        if average:
            return self.average_time
        else:
            return self.diff


class DetectionDeploySer(object):
    def __init__(self):
        self.input_size = 416

    def get_ckpt_data(self, ckpt_dir):
        names = {}
        if tf.gfile.IsDirectory(FLAGS.checkpoint_dir):
            checkpoint_dir = tf.train.latest_checkpoint(FLAGS.checkpoint_dir)
        else:
            checkpoint_dir = ckpt_dir

        catagory = os.path.join(ckpt_dir, 'category.txt')
        with open(catagory, 'r') as data:
            for ID, name in enumerate(data):
                names[ID] = name.strip('\n')

        return checkpoint_dir, names

    def load_model(self, model_path, paras):
        FLAGS.checkpoint_dir = model_path
        if FLAGS.checkpoint_dir == ' ':
            raise IOError('Error: model_path not found.\n')

        with tf.name_scope('input'):
            self.input_data = tf.placeholder(
                dtype=tf.float32, name='input_data'
            )

        from core.yolov3 import YOLOV3
        model = YOLOV3(self.input_data, trainable=False)
        print(model.conv_sbbox, model.conv_mbbox, model.conv_lbbox)

        self._pred_sbbox, self._pred_mbbox, self._pred_lbbox = \
            model.pred_sbbox, model.pred_mbbox, model.pred_lbbox

        self.sess = tf.Session(config=tf.ConfigProto(allow_soft_placement=True))
        init = tf.global_variables_initializer()
        self.sess.run(init)

        ckpt, self.classes = self.get_ckpt_data(FLAGS.checkpoint_dir)
        self.num_classes = len(self.classes)
        saver = tf.train.Saver()
        saver.restore(self.sess, ckpt)

        print('\n\nLoaded network {:s}'.format(FLAGS.checkpoint_dir))

    def vis_detections(self, image, bboxes, show_label=True):
        """
        bboxes: [x_min, y_min, x_max, y_max,
        probability, cls_id] format coordinates.
        """
        image_h, image_w, _ = image.shape
        classes = self.classes
        hsv_tuples = [(1.0 * x / self.num_classes, 1., 1.)
                      for x in range(self.num_classes)]
        colors = list(map(lambda x: colorsys.hsv_to_rgb(*x), hsv_tuples))
        colors = list(map(lambda x: (
            int(x[0] * 255), int(x[1] * 255), int(x[2] * 255)), colors))

        random.seed(0)
        random.shuffle(colors)
        random.seed(None)

        detect_result = []

        print("bboxes: ", bboxes)

        for i, bbox in enumerate(bboxes):
            coor = np.array(bbox[:4], dtype=np.int32)
            fontScale = 0.5
            score = bbox[4]
            class_ind = int(bbox[5])
            bbox_color = colors[class_ind]
            bbox_thick = int(0.6 * (image_h + image_w) / 600)
            c1, c2 = (coor[0], coor[1]), (coor[2], coor[3])
            cv2.rectangle(image, c1, c2, bbox_color, bbox_thick)

            res = {
                "probability": round(float(score), 4),
                "left": int(round(coor[0])),
                "top": int(round(coor[1])),
                "width": int(round(coor[2] - coor[0])),
                "height": int(round(coor[3] - coor[1]))
            }

            if show_label:
                bbox_mess = '%s: %.2f' % (classes[class_ind], score)
                print('predict_label:', classes[class_ind])
                res.update(label=classes[class_ind])

                t_size = cv2.getTextSize(
                    bbox_mess, 0, fontScale, thickness=bbox_thick // 2
                )[0]
                cv2.rectangle(
                    image, c1, (c1[0] + t_size[0], c1[1] - t_size[1] - 3),
                    bbox_color, -1)  # filled

                cv2.putText(
                    image, bbox_mess, (c1[0], c1[1] - 2),
                    cv2.FONT_HERSHEY_SIMPLEX,
                    fontScale, (0, 0, 0), bbox_thick // 2,
                    lineType=cv2.LINE_AA
                )

            detect_result.append(res)
        return image, detect_result

    def demo(self, sess, image_path):
        import core.utils as utils
        original_image = cv2.imread(image_path)
        original_image = cv2.cvtColor(original_image, cv2.COLOR_BGR2RGB)
        original_image_size = original_image.shape[:2]
        image_data = utils.image_preporcess(
            np.copy(original_image), [self.input_size, self.input_size]
        )
        image_data = image_data[np.newaxis, ...]

        timer = Timer()
        timer.tic()
        pred_sbbox, pred_mbbox, pred_lbbox = sess.run(
            [self._pred_sbbox, self._pred_mbbox, self._pred_lbbox],
            feed_dict={self.input_data: image_data}
        )
        num_classes = self.num_classes
        pred_bbox = np.concatenate(
            [np.reshape(pred_sbbox, (-1, 5 + num_classes)),
             np.reshape(pred_mbbox, (-1, 5 + num_classes)),
             np.reshape(pred_lbbox, (-1, 5 + num_classes))],
            axis=0
        )

        bboxes = utils.postprocess_boxes(
            pred_bbox, original_image_size, self.input_size, 0.3
        )
        bboxes = utils.nms(bboxes, 0.45, method='nms')

        timer.toc()
        print('Detection took {:.3f}s.'.format(
            timer.total_time
        ))
        image, detect_res = self.vis_detections(original_image, bboxes)

        yolov3_result = {
            "duration": int(round(1000 * timer.total_time)),
            "data": detect_res
        }

        return image, yolov3_result

    def inference(self, image_path):
        predict_ret = {}
        if image_path:
            print('Demo for {}'.format(image_path))
            predict_fig, predict_ret = self.demo(self.sess, image_path)
        return predict_ret


if __name__ == '__main__':
    model_path = "/home/hpcuser/ai_studio/tasks/" \
                 "69f02552-a641-11ea-b30b-b8599f2ba2e6/" \
                 "crise_yolov3_sg1_1/train_dir"
    image_path = '/home/hpcuser/crise/000005.jpg'
    parameters = None
    detection_deploy_ser = DetectionDeploySer()
    detection_deploy_ser.load_model(model_path, parameters)
    print(detection_deploy_ser.inference(image_path))
